@@ -1,14 +1,17 @@
+/*
+* (C) 2009 Jack Lloyd
+*
+* Distributed under the terms of the Botan license
+*/
+
 /*
 Decrypt files encrypted with the 'encrypt' example application.
 
-I'm being lazy and writing the output to stdout rather than stripping off the
-".enc" suffix and writing it there. So all diagnostics go to stderr so there is
-no confusion.
-
-Written by Jack Lloyd (lloyd@randombit.net) on August 5, 2002
-
-This file is in the public domain
+I'm being lazy and writing the output to stdout rather than stripping
+off the ".enc" suffix and writing it there. So all diagnostics go to
+stderr so there is no confusion.
 */
+
 #include <fstream>
 #include <iostream>
 #include <string>
@@ -93,22 +96,34 @@ int main(int argc, char* argv[])
          return 1;
          }
 
-      if(!have_block_cipher(algo))
+      const BlockCipher* cipher_proto = global_state().algorithm_factory().prototype_block_cipher(algo);
+
+      if(!cipher_proto)
          {
          std::cout << "Don't know about the block cipher \"" << algo << "\"\n";
          return 1;
          }
 
-      const u32bit key_len = max_keylength_of(algo);
-      const u32bit iv_len = block_size_of(algo);
+      const u32bit key_len = cipher_proto->maximum_keylength();
+      const u32bit iv_len = cipher_proto->block_size();
 
       std::auto_ptr<PBKDF> pbkdf(get_pbkdf("PBKDF2(SHA-1)"));
-      pbkdf->set_iterations(8192);
-      pbkdf->change_salt(b64_decode(salt_str));
 
-      SymmetricKey bc_key = pbkdf->derive_key(key_len, "BLK" + passphrase);
-      InitializationVector iv = pbkdf->derive_key(iv_len, "IVL" + passphrase);
-      SymmetricKey mac_key = pbkdf->derive_key(16, "MAC" + passphrase);
+      const u32bit PBKDF2_ITERATIONS = 8192;
+
+      SecureVector<byte> salt = b64_decode(salt_str);
+
+      SymmetricKey bc_key = pbkdf->derive_key(key_len, "BLK" + passphrase,
+                                              &salt[0], salt.size(),
+                                              PBKDF2_ITERATIONS);
+
+      InitializationVector iv = pbkdf->derive_key(iv_len, "IVL" + passphrase,
+                                                  &salt[0], salt.size(),
+                                                  PBKDF2_ITERATIONS);
+
+      SymmetricKey mac_key = pbkdf->derive_key(16, "MAC" + passphrase,
+                                               &salt[0], salt.size(),
+                                               PBKDF2_ITERATIONS);
 
       Pipe pipe(new Base64_Decoder,
                 get_cipher(algo + "/CBC", bc_key, iv, DECRYPTION),
